import json
import os
from pathlib import Path

from li_aws_helper.exceptions import handle_exception, AWSCredentialsNotFound

BASE_DIR = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))


class AWSConfig:
    _credentials = {}
    _credentials_file_name = f"{BASE_DIR}/credentials.json"
    region_name: str
    access_key: str
    secret_key: str
    session_token: str
    mfa_arn: str

    def __init__(self, load_config=None):
        if load_config:
            self.load_config()

    def check_configuration(self):
        file_exists = Path(self._credentials_file_name).exists()
        if not file_exists:
            raise AWSCredentialsNotFound()

    @handle_exception
    def load_config(self):
        self.check_configuration()
        with open(self._credentials_file_name, 'r') as file:
            credentials = json.load(file)
            self.region_name = credentials.get('region_name')
            self.access_key = credentials.get('access_key')
            self.secret_key = credentials.get('secret_key')
            self.session_token = credentials.get('session_token')
            self.mfa_arn = credentials.get('mfa_arn')

    def write_local_config(self):
        with open(self._credentials_file_name, 'w+') as file:
            file.write(json.dumps(self._credentials))

    def configure(self, region_name, access_key, secret_key, mfa_arn, session_token=None):
        self._credentials = {'region_name': region_name,
                             'access_key': access_key,
                             'secret_key': secret_key,
                             'mfa_arn': mfa_arn,
                             'session_token': session_token}
        self.write_local_config()



import typer

from li_aws_helper.aws_config import AWSConfig
from li_aws_helper.sts_client import refresh_token
from li_aws_helper.exceptions import handle_exception

app = typer.Typer()


@app.command()
@handle_exception
def refresh(token: str = typer.Option("", help="Token code")):
    refresh_token(token)
    message = typer.style("Tokens refreshed", fg=typer.colors.GREEN, bold=True)
    typer.echo(message)


@app.command()
@handle_exception
def config():
    region_name = typer.prompt("Region", default='us-east-1', show_default=True)
    access_key = typer.prompt("Access Key")
    secret_key = typer.prompt("Secret Key")
    mfa_arn = typer.prompt("MFA Identifier")

    AWSConfig().configure(region_name, access_key, secret_key, mfa_arn)
    message = typer.style("All Done!", fg=typer.colors.GREEN, bold=True)
    typer.echo(message)


if __name__ == "__main__":
    app()



import os
import boto3

from li_aws_helper.aws_config import AWSConfig


def refresh_token(token_code: str):
    aws_config = AWSConfig(load_config=True)

    session = boto3.session.Session(
        aws_access_key_id=aws_config.access_key,
        aws_secret_access_key=aws_config.secret_key
    )

    client = session.client(service_name='sts', region_name=aws_config.region_name)

    response = client.get_session_token(
        DurationSeconds=129600,
        SerialNumber=aws_config.mfa_arn,
        TokenCode=token_code)

    if response:
        credentials = response['Credentials']

        items = ['[default]',
                 f'aws_secret_access_key = {credentials["SecretAccessKey"]}',
                 f'aws_access_key_id = {credentials["AccessKeyId"]}',
                 f'aws_session_token = {credentials["SessionToken"]}']

        user_aws_credentials_file = f"{os.path.expanduser('~')}/.aws/credentials"
        with open(user_aws_credentials_file, 'w+') as file:
            items = map(lambda x: x + '\n', items)
            file.writelines(items)



from functools import wraps
from http import HTTPStatus

import typer


def handle_exception(func):
    @wraps(func)
    def wrapper(*args, **kw):
        try:
            return func(*args, **kw)
        except Exception as e:
            message = typer.style(f"Ops! We found an error: {str(e)}",
                                  fg=typer.colors.RED, bold=True)
            typer.echo(message)
    return wrapper


class Error(Exception):
    name = None
    description = None
    status = HTTPStatus.INTERNAL_SERVER_ERROR

    def __init__(self, *args: object, **kwargs) -> None:
        super().__init__(*args)
        self.name = kwargs.get("name")
        self.description = kwargs.get("description")
        self.status = kwargs.get("status")

    def __str__(self):
        return self.description

    def to_dict(self):
        return {
            'error': {
                'name': self.name,
                'description': self.description,
            }}


class AWSCredentialsNotFound(Error):
    def __init__(self, *args: object, **kwargs) -> None:
        super().__init__(*args, **kwargs)
        self.name = "credentials-not-found"
        self.description = "You must configurate your aws credentials at li before using aws commands." \
                           " Try: > li aws configure"
        self.status = HTTPStatus.NOT_FOUND
